@@ -22,10 +22,12 @@ import logging
 import gc
 import genshi
 import sys
+import re
 
 from config import LazygalConfig, LazygalWebgalConfig
 from config import USER_CONFIG_PATH, LazygalConfigDeprecated
 from sourcetree import SOURCEDIR_CONFIGFILE
+from pygexiv2 import GExiv2
 
 import make
 import pathutils
@@ -153,7 +155,7 @@ class SubgalBreak(make.MakeTask):
         try:
             while True:
                 subgal = subgals_it.next()
-                how_many_medias += subgal.source_dir.get_media_count()
+                how_many_medias += subgal.get_media_count()
                 galleries.append((subgal, subgal.medias))
                 if how_many_medias > self.webgal_dir.thumbs_per_page:
                     self.webgal_dir.add_index_page(subgals, galleries)
@@ -325,12 +327,33 @@ class WebalbumDir(make.FileMakeObject):
             # Directory did not exist, mark it as so
             self.stamp_delete()
 
+        tagfilters = self.config.getlist('webgal', 'filter-by-tag')
+
         self.medias = []
         self.sort_task = SubgalSort(self)
         self.sort_task.add_dependency(self.source_dir)
         for media in self.source_dir.medias:
             self.sort_task.add_dependency(media)
 
+            if len(tagfilters) > 0 and media.info() is not None:
+                # tag-filtering is requested
+                res = True
+                for tagf in tagfilters:
+                    # concatenate the list of tags as a string of words,
+                    # space-separated.  to ensure that we match the full
+                    # keyword and not only a subpart of it, we also surround
+                    # the matching pattern with spaces
+
+                    # we look for tag words, partial matches are not wanted
+                    regex = re.compile(r"\b" + tagf + r"\b")
+
+                    kwlist = ' '.join(media.info().get_keywords())
+                    if re.search(regex, kwlist) is None:
+                        res = False
+                        break
+                if res is False:
+                    continue
+
             if media.type == 'image':
                 media_task = WebalbumImageTask(self, media)
             elif media.type == 'video':
@@ -341,6 +364,13 @@ class WebalbumDir(make.FileMakeObject):
             self.medias.append(media_task)
             self.add_dependency(media_task)
 
+        if self.config.getboolean('webgal', 'dirzip')\
+                and self.get_media_count() > 1:
+            self.dirzip = genfile.WebalbumArchive(self)
+            self.add_dependency(self.dirzip)
+        else:
+            self.dirzip = None
+
         self.index_pages = []
 
         if not self.should_be_flattened():
@@ -438,13 +468,6 @@ class WebalbumDir(make.FileMakeObject):
 
         self.thumbs_per_page = self.config.getint('webgal', 'thumbs-per-page')
 
-        if self.config.getboolean('webgal', 'dirzip')\
-                and self.source_dir.get_media_count() > 1:
-            self.dirzip = genfile.WebalbumArchive(self)
-            self.add_dependency(self.dirzip)
-        else:
-            self.dirzip = None
-
         self.quality = self.config.getint('webgal', 'jpeg-quality')
         self.save_options = {}
         if self.config.getboolean('webgal', 'jpeg-optimize'):
@@ -530,6 +553,16 @@ class WebalbumDir(make.FileMakeObject):
             all_subgals.extend(subgal.get_all_subgals())
         return all_subgals
 
+    def get_media_count(self, media_type=None):
+        if media_type is None:
+            return len(self.medias)
+        else:
+            typed_media_count = 0
+            for mediatask in self.medias:
+                if mediatask.media.type == media_type:
+                    typed_media_count += 1
+            return typed_media_count
+
     def get_all_medias_tasks(self):
         all_medias = list(self.medias)  # We want a copy here.
         for subgal in self.subgals:
@@ -722,6 +755,7 @@ class Album(object):
             logging.getLogger().setLevel(logging.ERROR)
         if self.config.getboolean('runtime', 'debug'):
             logging.getLogger().setLevel(logging.DEBUG)
+            GExiv2.log_set_level(GExiv2.LogLevel.INFO)
 
         self.clean_dest = self.config.getboolean('global', 'clean-destination')
         self.force_gen_pages = self.config.getboolean('global', 'force-gen-pages')
